{
Created with Borland Delphi Version 3

Demo program to demonstrate the use of WP2COMM.DLL to communicate
with ITK controllers using the serial port.

Rem. The Test(Button) routines will only work with MC2000/MCX/Delta-PC controllers.
The C++Builder Demo has the full functionality for all controller types

Topics:
1. Initialize, open and close controller
2. Use of interface functions to execute wrapped venus-commands
3. Executing free defined venus-commands
4. Use of [Ctrl]+[B] to abort commands
	After initializing and opening the controller
	enter '100 100 0 rmove' in the edit field,
	set LinesExpected to 0, press Execute
	and then press [Ctrl]+[B] to abort the move
5. Demonstrating synchronous and asynchronous mode
}

unit UMain;

interface

uses
  Windows, Messages, SysUtils, Forms, Dialogs,
  StdCtrls, Classes, Controls, inifiles, Spin;

const
//Message used in asynchronous mode to notify UserWin
//that the controller replied to a venus-command.
//See function InitController(), parameters AsyncMsg and UserWin.
//Also used to signal a detected serial port BREAK
	WMU_ASYNCREPLYCOMPLETE=WM_USER+101;

	WP2_CONTROLLER_DEFAULT=1;
	WP2_CONTROLLER_DELTA=2;
	WP2_CONTROLLER_ALPHA=4;
	WP2_CONTROLLER_TAURUS=8;
	WP2_CONTROLLER_PEGASUS=16;
	WP2_CONTROLLER_ORION=32;
	WP2_CONTROLLER_CORVUS=64;

	function	InitController(CMode:DWORD;Axes:DWORD;ComPort:DWORD;Baudrate:DWORD;UserWin:HWND;AsyncMsg:UINT;Mode:DWORD)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	OpenController
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	ResetComm
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	CloseController
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	ExecuteCommand(Command:LPSTR;LinesExpected:DWORD;Reply:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetReply(pReply:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	AbortCommand
			:DWORD;stdcall;external 'Wp2Comm.dll';

	function	Calibrate
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	ClearParameterStack
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetAcceleration(Accel:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetAxisMode(A1,A2,A3,A4:LPDWORD)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetError(Error:LPDWORD)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetJoystickVelocity(JoyVel:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetLimits(A1Min,A1Max,A2Min,A2Max,A3Min,A3Max,A4Min,A4Max:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetParamsOnStack(Value:LPDWORD)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetPos(A1,A2,A3,A4:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetPosAxis(Axis:DWORD;Pos:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetStatus(Status:LPDWORD)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	GetVelocity(Vel:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	Identify(Id:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	JoystickDisable
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	JoystickEnable
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	MoveAbsolute(A1,A2,A3,A4:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	MoveAbsoluteAutoReply(A1,A2,A3,A4:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	MoveRelative(A1,A2,A3,A4:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	MoveRelativeAutoReply(A1,A2,A3,A4:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	RangeMeasure
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	SetAcceleration(Accel:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	SetAxisMode(A1,A2,A3,A4:DWORD)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	SetJoystickVelocity(JoyVel:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	SetOrigin
			:DWORD;stdcall;external 'Wp2Comm.dll';
	function	SetVelocity(Vel:LPSTR)
			:DWORD;stdcall;external 'Wp2Comm.dll';

type
  TFormMain = class(TForm)
    ButtonOpenController: TButton;
    Memo1: TMemo;
    ButtonCloseController: TButton;
    ButtonTest: TButton;
    CheckBoxCalRm: TCheckBox;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    EditCommand: TEdit;
    SpinEditLinesToReply: TSpinEdit;
    LabelLinesExpected: TLabel;
    GroupBoxInit: TGroupBox;
    ComboBoxPort: TComboBox;
    LabelPort: TLabel;
    Label2: TLabel;
    ComboBoxBaudrate: TComboBox;
    Label3: TLabel;
    SpinEditAxes: TSpinEdit;
    ButtonInit: TButton;
    ComboBoxController: TComboBox;
    Label4: TLabel;
    procedure ButtonInitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonOpenControllerClick(Sender: TObject);
    procedure EditCommandKeyPress(Sender: TObject; var Key: Char);
    procedure ButtonCloseControllerClick(Sender: TObject);
    procedure ButtonTestClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
	 Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
    procedure ComboBoxControllerChange(Sender: TObject);
  private
    { Private-Deklarationen }
	nAxes:Integer;
	nControllerMode:Integer;
	procedure	DoExecuteCommand;
	procedure WMUAsyncReplyComplete(var Msg:TMessage);message WMU_ASYNCREPLYCOMPLETE;
	function	DoTest:Boolean;
	procedure	SetupComboBoxBaudrate(nMode:Integer);
  public
    { Public-Deklarationen }
  end;

var
  FormMain: TFormMain;
	pReply:array[0..7]of PChar;
	dwValue:array[0..3]of DWORD;


implementation

{$R *.DFM}

procedure TFormMain.FormCreate(Sender: TObject);
var
	i:Integer;
	MyIniFile:TIniFile;
	strId:string;
begin
nControllerMode:=WP2_CONTROLLER_DEFAULT;
for i:=0 to 7 do
	pReply[i]:=StrAlloc(256);
ClientHeight:=Memo1.Top+Memo1.Height+Memo1.Left;
ClientWidth:=ButtonCloseController.Left+ButtonCloseController.Width+Memo1.Left;
ButtonOpenController.Enabled:=False;
ButtonCloseController.Enabled:=False;
ButtonTest.Enabled:=False;
EditCommand.Enabled:=False;
ComboBoxPort.ItemIndex:=1;
MyIniFile:=TIniFile.Create(ExtractFilePath(Application.ExeName)+'Wp2comm.ini');
strId:=MyIniFile.ReadString('Startup','CMode','Default');
MyIniFile.Free;
if (strId='Delta')or(strId='Delta-PC') then
	begin
	nControllerMode:=WP2_CONTROLLER_DELTA;
	ComboBoxController.ItemIndex:=1;
	SpinEditAxes.MaxValue:=4;
	SpinEditAxes.Value:=4;
	end
else if strId='Alpha' then
	begin
	nControllerMode:=WP2_CONTROLLER_ALPHA;
	ComboBoxController.ItemIndex:=2;
	SpinEditAxes.Value:=4;
	ButtonTest.Visible:=False;
	CheckBoxCalRm.Visible:=False;
	EditCommand.Text:='1 nversion';
	end
else if strId='Taurus' then
	begin
	nControllerMode:=WP2_CONTROLLER_TAURUS;
	ComboBoxController.ItemIndex:=3;
	SpinEditAxes.Value:=1;
	SpinEditAxes.Enabled:=False;
	ButtonTest.Visible:=False;
	CheckBoxCalRm.Visible:=False;
	EditCommand.Text:='1 nversion';
	end
else if strId='Pegasus' then
	begin
	nControllerMode:=WP2_CONTROLLER_PEGASUS;
	ComboBoxController.ItemIndex:=4;
	SpinEditAxes.Value:=4;
	ButtonTest.Visible:=False;
	CheckBoxCalRm.Visible:=False;
	EditCommand.Text:='1 nversion';
	end
else if strId='Orion' then
	begin
	nControllerMode:=WP2_CONTROLLER_ORION;
	ComboBoxController.ItemIndex:=5;
	SpinEditAxes.Value:=4;
	ButtonTest.Visible:=False;
	CheckBoxCalRm.Visible:=False;
	EditCommand.Text:='1 nversion';
	end
else if strId='Corvus' then
	begin
	nControllerMode:=WP2_CONTROLLER_CORVUS;
	ComboBoxController.ItemIndex:=6;
	SpinEditAxes.MaxValue:=3;
	SpinEditAxes.Value:=3;
	end
else
	begin
	nControllerMode:=WP2_CONTROLLER_DEFAULT;
	ComboBoxController.ItemIndex:=0;
	SpinEditAxes.MaxValue:=3;
	SpinEditAxes.Value:=3;
	end;
SetupComboBoxBaudrate(nControllerMode);
end;

procedure TFormMain.ButtonInitClick(Sender: TObject);
var
	dwError:DWORD;
	dwMode:DWORD;
	dwBaudrate:DWORD;
begin
case ComboBoxController.ItemIndex of
	1:nControllerMode:=WP2_CONTROLLER_DELTA;
	2:nControllerMode:=WP2_CONTROLLER_ALPHA;
	3:nControllerMode:=WP2_CONTROLLER_TAURUS;
	4:nControllerMode:=WP2_CONTROLLER_PEGASUS;
	5:nControllerMode:=WP2_CONTROLLER_ORION;
	6:nControllerMode:=WP2_CONTROLLER_CORVUS;
	else nControllerMode:=WP2_CONTROLLER_DEFAULT;
	end;
//set nControllerMode to 0 if you want Wp2comm.dll use the value in wp2comm.ini,[Startup] (CMode=Delta, CMode=Alpha or CMode=Default)
//dwMode:=2308+1048576;		//default value
dwMode:=1050884;
//dwMode:=$ffffffff;		//use Wp2Comm.ini
dwBaudrate:=StrToInt(ComboBoxBaudrate.Text);
nAxes:=SpinEditAxes.Value;
dwError:=InitController(nControllerMode,nAxes,ComboBoxPort.ItemIndex+1,dwBaudrate,Handle,WMU_ASYNCREPLYCOMPLETE,dwMode);
if dwError<>0 then
	begin
	ShowMessage(Format('Wp2CommInit(%d,%d,%d,%d,%d) returned Error %d',[nControllerMode,nAxes,ComboBoxPort.ItemIndex+1,dwBaudrate,dwMode,dwError]));
	exit;
	end;
ButtonOpenController.Enabled:=True;
ActiveControl:=ButtonOpenController;
end;

procedure TFormMain.ButtonOpenControllerClick(Sender: TObject);
var
	dwError:DWORD;
	strCaptionBk:string;
begin
ComboBoxController.Enabled:=False;
SpinEditAxes.Enabled:=False;
ComboBoxPort.Enabled:=False;
ComboBoxBaudrate.Enabled:=False;
strCaptionBk:=ButtonOpenController.Caption;
ButtonOpenController.Caption:='Please wait...';
dwError:=OpenController;
ButtonOpenController.Caption:=strCaptionBk;
if dwError<>0 then
	begin
	ShowMessage(Format('OpenController returned %d',[dwError]));
	ComboBoxController.Enabled:=True;
	SpinEditAxes.Enabled:=True;
	ComboBoxPort.Enabled:=True;
	if nControllerMode<>WP2_CONTROLLER_DELTA then
		ComboBoxBaudrate.Enabled:=True;
	EditCommand.Enabled:=False;
	if ButtonTest.Visible=True then
		ButtonTest.Enabled:=False;
	ButtonCloseController.Enabled:=False;
	ActiveControl:=ButtonOpenController;
	exit;
	end;
ButtonOpenController.Enabled:=False;
ButtonCloseController.Enabled:=True;
EditCommand.Enabled:=True;
Memo1.Lines.Clear;
if ButtonTest.Visible=True then
	begin
	ButtonTest.Enabled:=True;
	ActiveControl:=ButtonTest;
	end
else
	ActiveControl:=EditCommand;
end;

procedure TFormMain.DoExecuteCommand;
var
	dwError:DWORD;
begin
pReply[0][0]:=#0;
dwError:=ExecuteCommand(PChar(EditCommand.Text),SpinEditLinesToReply.Value,pReply[0]);
if dwError=0 then
	begin
	Memo1.Lines.Add(EditCommand.Text);
	if pReply[0][0]<>#0 then
		Memo1.Lines.Add(StrPas(pReply[0]));
	end
else
	begin
	ShowMessage(Format('ExecuteCommand(%s,%d,%s) returned =%d',
			[EditCommand.Text,SpinEditLinesToReply.Value,StrPas(pReply[0]),dwError]));
	end;
end;

procedure TFormMain.EditCommandKeyPress(Sender: TObject; var Key: Char);
begin
if Key=#13 then
	begin
	DoExecuteCommand;
	Key:=#0;
	end;
end;

procedure TFormMain.ButtonCloseControllerClick(Sender: TObject);
var
	dwError:DWORD;
begin
dwError:=CloseController;
if dwError=0 then
	begin
	ComboBoxController.Enabled:=True;
	if nControllerMode<>WP2_CONTROLLER_TAURUS then
		SpinEditAxes.Enabled:=True;
	ComboBoxPort.Enabled:=True;
	if nControllerMode<>WP2_CONTROLLER_DELTA then
		ComboBoxBaudrate.Enabled:=True;
	ButtonOpenController.Enabled:=True;
	if ButtonTest.Visible=True then
		ButtonTest.Enabled:=False;
	ButtonCloseController.Enabled:=False;
	EditCommand.Enabled:=False;
	ActiveControl:=ButtonOpenController;
	end;
end;

procedure TFormMain.ButtonTestClick(Sender: TObject);
begin
if DoTest=False then
	begin
	ResetComm;
	ShowMessage('Error processing test routine');
	end;
end;

function TFormMain.DoTest:Boolean;
var
	i:Integer;
	dwError:DWORD;
	dwMode:DWORD;
	dwBaudrate:DWORD;
begin
Result:=False;
pReply[0][0]:=#0;
pReply[1][0]:=#0;
pReply[2][0]:=#0;
pReply[3][0]:=#0;
dwValue[0]:=0;
dwValue[1]:=0;
dwValue[2]:=0;
dwValue[3]:=0;
Memo1.Lines.Add('Processing ClearParameterStack ...');
if ClearParameterStack>0 then
	exit;
Memo1.Lines.Add('Done');

if nControllerMode<>WP2_CONTROLLER_DELTA then
//GetParamsOnStack() not defined for DeltaPC
	begin
	Memo1.Lines.Add('Processing GetParamsOnStack ...');
	if GetParamsOnStack(@dwValue[0])>0 then
		exit;
	Memo1.Lines.Add(Format('%d',[dwValue[0]]));
	end;

Memo1.Lines.Add('Processing Identify ...');
if Identify(pReply[0])>0 then
	exit;
Memo1.Lines.Add(pReply[0]);

Memo1.Lines.Add('Processing GetStatus ...');
if GetStatus(@dwValue[0])>0 then
	exit;
Memo1.Lines.Add(Format('%d',[dwValue[0]]));

Memo1.Lines.Add('Processing GetError ...');
if GetError(@dwValue[0])>0 then
	exit;
Memo1.Lines.Add(Format('%d',[dwValue[0]]));

for i:=0 to nAxes-1 do
	begin
	if (i=nAxes-1)and(nAxes>1) then
		dwValue[i]:=0
	else
		dwValue[i]:=1;
	end;
Memo1.Lines.Add(Format('Processing SetAxisMode(%d,%d,%d,%d) ...',[dwValue[0],dwValue[1],dwValue[2],dwValue[3]]));
if SetAxisMode(dwValue[0],dwValue[1],dwValue[2],dwValue[3])>0 then
	exit;
Memo1.Lines.Add('Done');

Memo1.Lines.Add('Processing GetAxisMode ...');
if GetAxisMode(@dwValue[0],@dwValue[1],@dwValue[2],@dwValue[3])>0 then
	exit;
Memo1.Lines.Add(Format('%d %d %d %d',[dwValue[0],dwValue[1],dwValue[2],dwValue[3]]));

Memo1.Lines.Add('Processing SetVelocity(25,00) ...');
if SetVelocity('25,00')>0 then
	exit;
Memo1.Lines.Add('Done');

Memo1.Lines.Add('Processing GetVelocity ...');
if GetVelocity(pReply[0])>0 then
	exit;
Memo1.Lines.Add(pReply[0]);

Memo1.Lines.Add('Processing SetAcceleration(10,00) ...');
if SetAcceleration('10,00')>0 then
	exit;
Memo1.Lines.Add('Done');

Memo1.Lines.Add('Processing GetAcceleration ...');
if GetAcceleration(pReply[0])>0 then
	exit;
Memo1.Lines.Add(pReply[0]);

if nControllerMode<>WP2_CONTROLLER_DELTA then
//Joystick functions not defined for DeltaPC
	begin
	Memo1.Lines.Add('Processing JoystickEnable ...');
	if JoystickEnable>0 then
		exit;
	Memo1.Lines.Add('Done');

	Memo1.Lines.Add('Processing JoystickDisable ...');
	if JoystickDisable>0 then
		exit;
	Memo1.Lines.Add('Done');

	Memo1.Lines.Add('Processing SetJoystickVelocity(5,00) ...');
	if SetJoystickVelocity('5,00')>0 then
		exit;
	Memo1.Lines.Add('Done');

	Memo1.Lines.Add('Processing GetJoystickVelocity ...');
	if GetJoystickVelocity(pReply[0])>0 then
		exit;
	Memo1.Lines.Add(pReply[0]);
	end;

Memo1.Lines.Add('Processing SetOrigin ...');
if SetOrigin>0 then
	exit;
Memo1.Lines.Add('Done');

Memo1.Lines.Add('Processing MoveAbsolute(20,15,0,0) ...');
if MoveAbsolute('20','15','0','0')>0 then
	exit;
while True do
	begin
	if GetPos(pReply[0],pReply[1],pReply[2],pReply[3])>0 then
		exit;
	Memo1.Lines.Add(Format('Pos. %s %s %s %s',[pReply[0],pReply[1],pReply[2],pReply[3]]));
	if GetStatus(@dwValue[0])>0 then
		exit;
	dwValue[0]:=(dwValue[0])and(3);
	if (dwValue[0]=0)or(dwValue[0]=2) then
		break;
	end;
Memo1.Lines.Add('Done');
if GetPos(pReply[0],pReply[1],pReply[2],pReply[3])>0 then
	exit;
Memo1.Lines.Add(Format('Pos. %s %s %s %s',[pReply[0],pReply[1],pReply[2],pReply[3]]));

Memo1.Lines.Add('Processing MoveRelative(-5,-10,0,0) ...');
if MoveRelative('-5','-10','0','0')>0 then
	exit;
while True do
	begin
	if GetPos(pReply[0],pReply[1],pReply[2],pReply[3])>0 then
		exit;
	Memo1.Lines.Add(Format('Pos. %s %s %s %s',[pReply[0],pReply[1],pReply[2],pReply[3]]));
	if GetStatus(@dwValue[0])>0 then
		exit;
	dwValue[0]:=(dwValue[0])and(3);
	if (dwValue[0]=0)or(dwValue[0]=2) then
		break;
	end;
Memo1.Lines.Add('Done');
if GetPos(pReply[0],pReply[1],pReply[2],pReply[3])>0 then
	exit;
Memo1.Lines.Add(Format('Pos. %s %s %s %s',[pReply[0],pReply[1],pReply[2],pReply[3]]));

Memo1.Lines.Add('Processing MoveAbsoluteAutoReply(15,20,0,0) ...');
if MoveAbsoluteAutoReply('15','20','0','0')>0 then
	exit;
if GetPos(pReply[0],pReply[1],pReply[2],pReply[3])>0 then
	exit;
Memo1.Lines.Add(Format('Pos. %s %s %s %s',[pReply[0],pReply[1],pReply[2],pReply[3]]));

Memo1.Lines.Add('Processing MoveRelativeAutoReply(-10,-10,0,0) ...');
if MoveRelativeAutoReply('-10','-10','0','0')>0 then
	exit;
if GetPos(pReply[0],pReply[1],pReply[2],pReply[3])>0 then
	exit;
Memo1.Lines.Add(Format('Pos. %s %s %s %s',[pReply[0],pReply[1],pReply[2],pReply[3]]));

Memo1.Lines.Add('Processing GetPosAxis ...');
if GetPosAxis(1,pReply[0])>0 then
	exit;
Memo1.Lines.Add(pReply[0]);

Memo1.Lines.Add('Processing GetPos ...');
if GetPos(pReply[0],pReply[1],pReply[2],pReply[3])>0 then
	exit;
Memo1.Lines.Add(Format('%s %s %s %s',[pReply[0],pReply[1],pReply[2],pReply[3]]));

if CheckBoxCalRm.Checked=True then
	begin
	Memo1.Lines.Add('Processing Calibrate ...');
	if Calibrate>0 then
		exit;
	Memo1.Lines.Add('Done');
	Memo1.Lines.Add('Processing RangeMeasure ...');
	if RangeMeasure>0 then
		exit;
	Memo1.Lines.Add('Done');
	end;

Memo1.Lines.Add('Processing GetLimits ...');
if GetLimits(pReply[0],pReply[1],pReply[2],pReply[3],pReply[4],pReply[5],pReply[6],pReply[7])>0 then
	exit;
Memo1.Lines.Add(Format('Axis 1: %s | %s',[pReply[0],pReply[1]]));
if nAxes>1 then
	Memo1.Lines.Add(Format('Axis 2: %s | %s',[pReply[2],pReply[3]]));
if nAxes>2 then
	Memo1.Lines.Add(Format('Axis 3: %s | %s',[pReply[4],pReply[5]]));
if nAxes>3 then
	Memo1.Lines.Add(Format('Axis 4: %s | %s',[pReply[6],pReply[7]]));

//switch to asynchronous mode
//dwMode:=2564+1048576;
dwMode:=1051140;
dwBaudrate:=StrToInt(ComboBoxBaudrate.Text);
dwError:=InitController(nControllerMode,nAxes,ComboBoxPort.ItemIndex+1,dwBaudrate,Handle,WMU_ASYNCREPLYCOMPLETE,dwMode);
if dwError<>0 then
	begin
	ShowMessage(Format('InitController(%d,%d,%d,%d,%d) returned Error %d',[nControllerMode,nAxes,ComboBoxPort.ItemIndex+1,dwBaudrate,dwMode,dwError]));
	exit;
	end;
Memo1.Lines.Add('Processing GetError in asynchronous mode ...');
//execute GetError in asynchronous mode
GetError(@dwValue[0]);

//switch back to synchronous mode
dwMode:=1050884;
dwBaudrate:=StrToInt(ComboBoxBaudrate.Text);
dwError:=InitController(nControllerMode,nAxes,ComboBoxPort.ItemIndex+1,dwBaudrate,Handle,WMU_ASYNCREPLYCOMPLETE,dwMode);
if dwError<>0 then
	begin
	ShowMessage(Format('InitController(%d,%d,%d,%d,%d) returned Error %d',[nControllerMode,nAxes,ComboBoxPort.ItemIndex+1,dwBaudrate,dwMode,dwError]));
	exit;
	end;
Result:=True;
end;

procedure TFormMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
if ((Key=Word('B'))and(Shift=[ssCtrl])) then
	begin
	AbortCommand;
	Key:=0;
	end;
end;

procedure TFormMain.WMUAsyncReplyComplete(var Msg:TMessage);
var
	pReply:PChar;
	dwError:DWORD;
begin
if Msg.wParam=21 then
	begin
	ResetComm;
	ShowMessage('BREAK detected!');
	end
else
	begin
	pReply:=StrAlloc(256);
	if pReply=nil then
		exit;
	pReply[0]:=#0;
	dwError:=GetReply(pReply);
	if dwError=0 then
		Memo1.Lines.Add(pReply)
	else
		ShowMessage(Format('Controller reply error >%d<',[dwError]));
	ActiveControl:=EditCommand;
	StrDispose(pReply);
	end;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
	i:Integer;
begin
if ButtonCloseController.Enabled=True then
	CloseController;
for i:=0 to 7 do
	StrDispose(pReply[i]);
end;

procedure TFormMain.FormResize(Sender: TObject);
begin
Memo1.Width:=ClientWidth-2*Memo1.Left;
end;

procedure TFormMain.ComboBoxControllerChange(Sender: TObject);
begin
if ComboBoxController.ItemIndex=1 then
	begin
	nControllerMode:=WP2_CONTROLLER_DELTA;
	SpinEditAxes.MaxValue:=4;
	SpinEditAxes.Value:=4;
	SpinEditAxes.Enabled:=True;
	ButtonTest.Visible:=True;
	CheckBoxCalRm.Visible:=True;
	EditCommand.Text:='version';
	end
else if ComboBoxController.ItemIndex=2 then
	begin
	nControllerMode:=WP2_CONTROLLER_ALPHA;
	SpinEditAxes.MaxValue:=16;
	SpinEditAxes.Value:=4;
	SpinEditAxes.Enabled:=True;
	ButtonTest.Visible:=False;
	CheckBoxCalRm.Visible:=False;
	EditCommand.Text:='1 nversion';
	end
else if ComboBoxController.ItemIndex=3 then
	begin
	nControllerMode:=WP2_CONTROLLER_TAURUS;
	SpinEditAxes.MaxValue:=1;
	SpinEditAxes.Value:=1;
	SpinEditAxes.Enabled:=False;
	ButtonTest.Visible:=False;
	CheckBoxCalRm.Visible:=False;
	EditCommand.Text:='1 nversion';
	end
else if ComboBoxController.ItemIndex=4 then
	begin
	nControllerMode:=WP2_CONTROLLER_PEGASUS;
	SpinEditAxes.MaxValue:=100;
	SpinEditAxes.Value:=4;
	SpinEditAxes.Enabled:=True;
	ButtonTest.Visible:=False;
	CheckBoxCalRm.Visible:=False;
	EditCommand.Text:='1 nversion';
	end
else if ComboBoxController.ItemIndex=5 then
	begin
	nControllerMode:=WP2_CONTROLLER_ORION;
	SpinEditAxes.MaxValue:=100;
	SpinEditAxes.Value:=4;
	SpinEditAxes.Enabled:=True;
	ButtonTest.Visible:=False;
	CheckBoxCalRm.Visible:=False;
	EditCommand.Text:='1 nversion';
	end
else if ComboBoxController.ItemIndex=6 then
	begin
	nControllerMode:=WP2_CONTROLLER_CORVUS;
	SpinEditAxes.MaxValue:=3;
	SpinEditAxes.Value:=3;
	SpinEditAxes.Enabled:=True;
	ButtonTest.Visible:=True;
	CheckBoxCalRm.Visible:=True;
	EditCommand.Text:='version';
	end
else
	begin
	nControllerMode:=WP2_CONTROLLER_DEFAULT;
	SpinEditAxes.MaxValue:=3;
	SpinEditAxes.Value:=3;
	ButtonTest.Visible:=True;
	CheckBoxCalRm.Visible:=True;
	EditCommand.Text:='version';
	end;
SetupComboBoxBaudrate(nControllerMode);
end;

procedure TFormMain.SetupComboBoxBaudrate(nMode:Integer);
begin
ComboBoxBaudrate.Items.Clear();
ComboBoxBaudrate.Items.Add('19200');
ComboBoxBaudrate.Items.Add('9600');
ComboBoxBaudrate.Items.Add('4800');
ComboBoxBaudrate.Items.Add('2400');
ComboBoxBaudrate.Items.Add('1200');
case(nMode) of
WP2_CONTROLLER_DELTA:
	begin
	ComboBoxBaudrate.ItemIndex:=ComboBoxBaudrate.Items.IndexOf('9600');
	ComboBoxBaudrate.Enabled:=false;
	end;
WP2_CONTROLLER_CORVUS:
	begin
	ComboBoxBaudrate.Items.Insert(0,'57600');
	ComboBoxBaudrate.Items.Insert(0,'115200');
	ComboBoxBaudrate.ItemIndex:=ComboBoxBaudrate.Items.IndexOf('115200');
	ComboBoxBaudrate.Enabled:=True;
	end;
WP2_CONTROLLER_ALPHA,
WP2_CONTROLLER_TAURUS,
WP2_CONTROLLER_PEGASUS,
WP2_CONTROLLER_ORION,
WP2_CONTROLLER_DEFAULT:
	begin
	ComboBoxBaudrate.ItemIndex:=ComboBoxBaudrate.Items.IndexOf('9600');
	ComboBoxBaudrate.Enabled:=True;
	end;
else
	begin
	ComboBoxBaudrate.ItemIndex:=ComboBoxBaudrate.Items.IndexOf('19200');
	ComboBoxBaudrate.Enabled:=True;
	end;
	end;	//case
end;

end.

